{% extends 'admin/solo/change_form.html' %}
{% load i18n %}

{% block object-tools-items %}
    <li>
      <a href="{% url 'metadata:eherkenning' %}">{% trans "View SAML metadata (XML)" %}</a>
    </li>
    <li>
      <a href="{% url 'metadata:eh-dienstcatalogus' %}">
        {% trans "View service catalogue metadata (XML)" %}
      </a>
    </li>
{{ block.super }}
{% endblock %}
