# Generated by Django 4.2.4 on 2023-09-01 09:23

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):
    dependencies = [
        ("digid_eherkenning", "0004_alter_eherkenningconfiguration_loa"),
    ]

    operations = [
        migrations.AlterField(
            model_name="eherkenningconfiguration",
            name="eh_service_instance_uuid",
            field=models.UUIDField(
                default=uuid.uuid4,
                help_text="UUID of the eHerkenning service instance. Once entered into catalogues, changing the value is a manual process.",
                verbose_name="eHerkenning service instance UUID",
            ),
        ),
        migrations.AlterField(
            model_name="eherkenningconfiguration",
            name="eidas_service_instance_uuid",
            field=models.UUIDField(
                default=uuid.uuid4,
                help_text="UUID of the eIDAS service instance. Once entered into catalogues, changing the value is a manual process.",
                verbose_name="eIDAS service instance UUID",
            ),
        ),
    ]
