.digidmock-anchor-placeholder {
    cursor: pointer;
    cursor: not-allowed;
}

.digidmock-notification {
    /* copied from the error message */
    margin-bottom: 15px;
    margin-top: 15px;

    background-image: url(assets/error.svg);
    background-color: #f9dfdd;
    background-position: 10px 10px;
    background-repeat: no-repeat;
    background-size: 21px 21px;
    min-height: 20px;
    padding-left: 38px;
    padding-right: 5px;
    padding-top: 10px;
    padding-bottom: 10px;
    width: auto;
    border-radius: 4px;
    position: relative;
}


.digidmock-error {
    display: none;
}

.digidmock-show {
    display: block;
}


