{% load static %}
<!DOCTYPE html>
<html>
    <head>
        <title>U wordt doorgestuurd</title>
        <link rel="stylesheet" type="text/css" href="{% static 'digid_eherkenning/login_submit.css' %}">
    </head>
    <body>
        <script src="{% static 'digid_eherkenning/login_submit.js' %}"></script>
        <form class="digid-eherkenning-submit-form" method="post" action="{{ url }}" name="login">
            {{ form.as_p }}
            <input type="submit" value="U wordt doorgestuurd" />
        </form>
    </body>
</html>
