# Generated by Django 4.2.10 on 2024-03-04 13:08

from django.db import migrations, models
import digid_eherkenning.models.eherkenning


class Migration(migrations.Migration):

    dependencies = [
        ("digid_eherkenning", "0006_digidconfiguration_metadata_file_source_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="eherkenningconfiguration",
            name="service_description_url",
            field=models.URLField(
                default="",
                help_text="The URL where the service description can be found.",
                max_length=255,
                verbose_name="service description URL",
            ),
        ),
        migrations.AlterField(
            model_name="eherkenningconfiguration",
            name="eh_requested_attributes",
            field=models.JSONField(
                default=digid_eherkenning.models.eherkenning.get_default_requested_attributes_eherkenning,
                help_text="A list of additional requested attributes. A single requested attribute can be a string (the name of the attribute) or an object with keys 'name' and 'required', where 'name' is a string and 'required' a boolean'.",
                verbose_name="requested attributes",
            ),
        ),
        migrations.AlterField(
            model_name="eherkenningconfiguration",
            name="eidas_requested_attributes",
            field=models.JSONField(
                default=digid_eherkenning.models.eherkenning.get_default_requested_attributes_eidas,
                help_text="A list of additional requested attributes. A single requested attribute can be a string (the name of the attribute) or an object with keys 'name' and 'required', where 'name' is a string and 'required' a boolean'.",
                verbose_name="requested attributes",
            ),
        ),
    ]
